import { inject, Injectable } from "@angular/core";
import { SdServiceEventListenerBase } from "@simplysm/sd-service-common";
import { SdServiceClientFactoryProvider } from "../../providers/sd-service-client-factory.provider";
import { ObjectUtils, Wait } from "@simplysm/sd-core-common";
import { $signal } from "../../utils/bindings/$signal";
import { $computed } from "../../utils/bindings/$computed";
import * as i0 from "@angular/core";
export class SdSharedDataProvider {
    constructor() {
        this._sdServiceFactory = inject(SdServiceClientFactoryProvider);
        this._infoMap = new Map();
        this.loadingCount = 0;
    }
    register(name, getter) {
        this._infoMap.set(name, { getter });
    }
    async emitAsync(name, changeKeys) {
        const info = this._infoMap.get(name);
        if (!info)
            throw new Error(`'${name}'에 대한 공유데이터 정보가 없습니다.`);
        await this._sdServiceFactory
            .get(info.getter.serviceKey)
            .emitAsync(SdSharedDataChangeEvent, (item) => item.name === name && ObjectUtils.equal(item.filter, info.getter.filter), changeKeys);
    }
    async wait() {
        await Wait.until(() => this.loadingCount <= 0);
    }
    getSignal(name) {
        const info = this._infoMap.get(name);
        if (!info)
            throw new Error(`'${name}'에 대한 공유데이터 정보가 없습니다.`);
        //-- listener
        if (info.listenerKey == null) {
            info.listenerKey = void this._sdServiceFactory.get(info.getter.serviceKey).addEventListenerAsync(SdSharedDataChangeEvent, {
                name,
                filter: info.getter.filter,
            }, async (changeKeys) => {
                await this._loadDataAsync(name, changeKeys);
            });
        }
        //-- data
        if (!info.signal) {
            info.signal = $signal([]);
            const computedMap = $computed(() => info.signal().toMap((item) => item.__valueKey));
            info.signal["$get"] = (key) => computedMap().get(key);
            void this._loadDataAsync(name);
        }
        return info.signal;
    }
    async _loadDataAsync(name, changeKeys) {
        this.loadingCount++;
        try {
            const info = this._infoMap.get(name);
            if (!info)
                throw new Error(`'${name}'에 대한 공유데이터 로직 정보가 없습니다.`);
            if (!info.signal)
                throw new Error(`'${name}'에 대한 공유데이터 저장소가 없습니다.`);
            const resData = await info.getter.getDataAsync(changeKeys);
            if (!changeKeys) {
                info.signal.set(this._ordering(resData, info.getter.orderBy));
            }
            else {
                info.signal.update((v) => {
                    // changeKeys에 있는것 전부 삭제
                    const r = v.filter(item => !changeKeys.includes(item.__valueKey));
                    // changeKeys로 검색한 결과물인, resData를 다시 입력
                    r.push(...resData);
                    // 재정렬
                    return this._ordering(r, info.getter.orderBy);
                });
            }
            this.loadingCount--;
        }
        catch (err) {
            this.loadingCount--;
            throw err;
        }
    }
    _ordering(data, orderByList) {
        let result = [...data];
        for (const orderBy of orderByList.reverse()) {
            if (orderBy[1] === "desc") {
                result = result.orderByDesc((item) => orderBy[0](item));
            }
            else {
                result = result.orderBy((item) => orderBy[0](item));
            }
        }
        return result;
    }
    static { this.ɵfac = function SdSharedDataProvider_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdSharedDataProvider)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdSharedDataProvider, factory: SdSharedDataProvider.ɵfac, providedIn: "root" }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdSharedDataProvider, [{
        type: Injectable,
        args: [{ providedIn: "root" }]
    }], null, null); })();
export class SdSharedDataChangeEvent extends SdServiceEventListenerBase {
}
//# sourceMappingURL=sd-shared-data.provider.js.map