import { ChangeDetectionStrategy, Component, ElementRef, input, viewChild, ViewEncapsulation, } from "@angular/core";
import jsbarcode from "jsbarcode";
import { $effect } from "../utils/bindings/$effect";
import * as i0 from "@angular/core";
const _c0 = ["svgEl"];
/**
 * jsbarcode 라이브러리를 사용하여 바코드를 렌더링하는 Angular 컴포넌트입니다.
 * 이 컴포넌트는 구성 가능한 속성을 가진 SVG 바코드를 생성합니다.
 */
export class SdBarcodeControl {
    /**
     * 컴포넌트를 초기화하고 입력 속성이 변경될 때마다
     * 바코드를 렌더링하는 효과를 설정합니다.
     */
    constructor() {
        /**
         * 바코드로 인코딩될 값입니다.
         * 이것은 바코드로 표현될 데이터입니다.
         */
        this.value = input();
        /**
         * 바코드의 유형/형식입니다.
         * 기본값은 "code128"이지만, jsbarcode에서 지원하는 다른 형식으로 변경할 수 있습니다.
         */
        this.type = input("code128");
        /**
         * 바코드의 각 라인 너비입니다.
         * 기본값은 1입니다.
         */
        this.lineWidth = input(1);
        /**
         * 바코드의 높이(픽셀 단위)입니다.
         * 기본값은 58입니다.
         */
        this.height = input(58);
        /**
         * 바코드가 렌더링될 템플릿의 SVG 요소에 대한 참조입니다.
         */
        this.svgElRef = viewChild.required("svgEl", { read: ElementRef });
        $effect(() => {
            jsbarcode(this.svgElRef().nativeElement, this.value() ?? "", {
                marginTop: -5,
                marginBottom: -5,
                format: this.type(),
                width: this.lineWidth(),
                height: this.height(),
                displayValue: false,
            });
        });
    }
    static { this.ɵfac = function SdBarcodeControl_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdBarcodeControl)(); }; }
    static { this.ɵcmp = /*@__PURE__*/ i0.ɵɵdefineComponent({ type: SdBarcodeControl, selectors: [["sd-barcode"]], viewQuery: function SdBarcodeControl_Query(rf, ctx) { if (rf & 1) {
            i0.ɵɵviewQuerySignal(ctx.svgElRef, _c0, 5, ElementRef);
        } if (rf & 2) {
            i0.ɵɵqueryAdvance();
        } }, inputs: { value: [1, "value"], type: [1, "type"], lineWidth: [1, "lineWidth"], height: [1, "height"] }, decls: 2, vars: 0, consts: [["svgEl", ""]], template: function SdBarcodeControl_Template(rf, ctx) { if (rf & 1) {
            i0.ɵɵnamespaceSVG();
            i0.ɵɵelement(0, "svg", null, 0);
        } }, encapsulation: 2, changeDetection: 0 }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdBarcodeControl, [{
        type: Component,
        args: [{
                selector: "sd-barcode",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                standalone: true,
                imports: [],
                template: `
    <svg #svgEl></svg>
  `,
            }]
    }], () => [], null); })();
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassDebugInfo(SdBarcodeControl, { className: "SdBarcodeControl", filePath: "packages/sd-angular/src/controls/sd-barcode.control.ts", lineNumber: 26 }); })();
//# sourceMappingURL=sd-barcode.control.js.map