import { ChangeDetectionStrategy, Component, ElementRef, input, model, viewChild, ViewEncapsulation, } from "@angular/core";
import { $effect } from "../utils/bindings/$effect";
import { setupInvalid } from "../utils/setups/setup-invalid";
import { transformBoolean } from "../utils/type-tramsforms";
import * as i0 from "@angular/core";
const _c0 = ["editorEl"];
export class SdContentEditorControl {
    constructor() {
        this.value = model();
        this.disabled = input(false, { transform: transformBoolean });
        this.readonly = input(false, { transform: transformBoolean });
        this.required = input(false, { transform: transformBoolean });
        this.validatorFn = input();
        this.inline = input(false, { transform: transformBoolean });
        this.inset = input(false, { transform: transformBoolean });
        this.size = input();
        this.editorStyle = input();
        this.editorElRef = viewChild.required("editorEl", {
            read: ElementRef,
        });
        setupInvalid(() => {
            const errorMessages = [];
            if (this.value() == null && this.required()) {
                errorMessages.push("값을 입력하세요.");
            }
            else if (this.validatorFn()) {
                const message = this.validatorFn()(this.value());
                if (message !== undefined) {
                    errorMessages.push(message);
                }
            }
            return errorMessages.join("\r\n");
        });
        $effect(() => {
            const innerHTML = this.editorElRef().nativeElement.innerHTML;
            if (innerHTML !== this.value()) {
                this.editorElRef().nativeElement.innerHTML = this.value() ?? "";
            }
        });
    }
    onEditorFocus() {
        if (!this.readonly()) {
            const selection = window.getSelection();
            selection.removeAllRanges();
            const range = document.createRange();
            range.selectNodeContents(this.editorElRef().nativeElement);
            selection.setPosition(this.editorElRef().nativeElement, range.endOffset);
        }
    }
    onInput() {
        const editorEl = this.editorElRef().nativeElement;
        let value;
        if (editorEl.innerHTML === "") {
            value = undefined;
        }
        else {
            value = editorEl.innerHTML;
        }
        this.value.set(value);
    }
    static { this.ɵfac = function SdContentEditorControl_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdContentEditorControl)(); }; }
    static { this.ɵcmp = /*@__PURE__*/ i0.ɵɵdefineComponent({ type: SdContentEditorControl, selectors: [["sd-content-editor"]], viewQuery: function SdContentEditorControl_Query(rf, ctx) { if (rf & 1) {
            i0.ɵɵviewQuerySignal(ctx.editorElRef, _c0, 5, ElementRef);
        } if (rf & 2) {
            i0.ɵɵqueryAdvance();
        } }, hostVars: 5, hostBindings: function SdContentEditorControl_HostBindings(rf, ctx) { if (rf & 2) {
            i0.ɵɵattribute("sd-disabled", ctx.disabled())("sd-readonly", ctx.readonly())("sd-inline", ctx.inline())("sd-inset", ctx.inset())("sd-size", ctx.size());
        } }, inputs: { value: [1, "value"], disabled: [1, "disabled"], readonly: [1, "readonly"], required: [1, "required"], validatorFn: [1, "validatorFn"], inline: [1, "inline"], inset: [1, "inset"], size: [1, "size"], editorStyle: [1, "editorStyle"] }, outputs: { value: "valueChange" }, decls: 2, vars: 4, consts: [["editorEl", ""], [1, "_editor", 3, "input", "focus", "title"]], template: function SdContentEditorControl_Template(rf, ctx) { if (rf & 1) {
            const _r1 = i0.ɵɵgetCurrentView();
            i0.ɵɵelementStart(0, "div", 1, 0);
            i0.ɵɵlistener("input", function SdContentEditorControl_Template_div_input_0_listener() { i0.ɵɵrestoreView(_r1); return i0.ɵɵresetView(ctx.onInput()); })("focus", function SdContentEditorControl_Template_div_focus_0_listener() { i0.ɵɵrestoreView(_r1); return i0.ɵɵresetView(ctx.onEditorFocus()); });
            i0.ɵɵelementEnd();
        } if (rf & 2) {
            i0.ɵɵstyleMap(ctx.editorStyle());
            i0.ɵɵproperty("title", ctx.value());
            i0.ɵɵattribute("contenteditable", !ctx.disabled() && !ctx.readonly());
        } }, styles: ["sd-content-editor{display:block;position:relative}sd-content-editor>._editor{display:block;width:100%;padding:var(--gap-sm) var(--gap-default);border:1px solid transparent;font-size:var(--font-size-default);font-family:var(--font-family);font-variant-numeric:tabular-nums;line-height:var(--line-height);color:var(--text-trans-default);white-space:pre-wrap;background:var(--theme-secondary-lightest);border:1px solid var(--trans-lighter);border-radius:var(--border-radius-default);min-height:calc(var(--gap-sm) * 2 + var(--font-size-default) * var(--line-height-strip-unit) + 2px)}sd-content-editor>._editor:focus{outline:none;border-color:var(--theme-primary-default)}sd-content-editor[sd-disabled=true]>._editor{background:var(--theme-grey-lightest);color:var(--text-trans-light)}sd-content-editor[sd-size=sm]>._editor{padding:var(--gap-xs) var(--gap-sm);min-height:calc(var(--gap-xs) * 2 + var(--font-size-default) * var(--line-height-strip-unit) + 2px)}sd-content-editor[sd-size=lg]>._editor{padding:var(--gap-default) var(--gap-lg);min-height:calc(var(--gap-default) * 2 + var(--font-size-default) * var(--line-height-strip-unit) + 2px)}sd-content-editor[sd-inline=true]{display:inline-block}sd-content-editor[sd-inline=true]>._editor{display:inline-block;width:auto;vertical-align:top}sd-content-editor[sd-inset=true]>._editor{border:none;border-radius:0}sd-content-editor[sd-inset=true][sd-disabled=true]>._editor{background:var(--control-color)!important;color:var(--text-trans-default)}sd-content-editor[sd-inset=true]>._editor{min-height:calc(var(--gap-sm) * 2 + var(--font-size-default) * var(--line-height-strip-unit))}sd-content-editor[sd-inset=true][sd-size=sm]>._editor{min-height:calc(var(--gap-xs) * 2 + var(--font-size-default) * var(--line-height-strip-unit))}sd-content-editor[sd-inset=true][sd-size=lg]>._editor{min-height:calc(var(--gap-default) * 2 + var(--font-size-default) * var(--line-height-strip-unit))}sd-content-editor[sd-inset=true]>._editor:focus{outline:1px solid var(--theme-primary-default);outline-offset:-1px}\n"], encapsulation: 2, changeDetection: 0 }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdContentEditorControl, [{
        type: Component,
        args: [{ selector: "sd-content-editor", changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, standalone: true, imports: [], template: `
    <div
      #editorEl
      class="_editor"
      (input)="onInput()"
      [attr.contenteditable]="!disabled() && !readonly()"
      [title]="value()"
      [style]="editorStyle()"
      (focus)="onEditorFocus()"
    ></div>
  `, host: {
                    "[attr.sd-disabled]": "disabled()",
                    "[attr.sd-readonly]": "readonly()",
                    "[attr.sd-inline]": "inline()",
                    "[attr.sd-inset]": "inset()",
                    "[attr.sd-size]": "size()",
                }, styles: ["sd-content-editor{display:block;position:relative}sd-content-editor>._editor{display:block;width:100%;padding:var(--gap-sm) var(--gap-default);border:1px solid transparent;font-size:var(--font-size-default);font-family:var(--font-family);font-variant-numeric:tabular-nums;line-height:var(--line-height);color:var(--text-trans-default);white-space:pre-wrap;background:var(--theme-secondary-lightest);border:1px solid var(--trans-lighter);border-radius:var(--border-radius-default);min-height:calc(var(--gap-sm) * 2 + var(--font-size-default) * var(--line-height-strip-unit) + 2px)}sd-content-editor>._editor:focus{outline:none;border-color:var(--theme-primary-default)}sd-content-editor[sd-disabled=true]>._editor{background:var(--theme-grey-lightest);color:var(--text-trans-light)}sd-content-editor[sd-size=sm]>._editor{padding:var(--gap-xs) var(--gap-sm);min-height:calc(var(--gap-xs) * 2 + var(--font-size-default) * var(--line-height-strip-unit) + 2px)}sd-content-editor[sd-size=lg]>._editor{padding:var(--gap-default) var(--gap-lg);min-height:calc(var(--gap-default) * 2 + var(--font-size-default) * var(--line-height-strip-unit) + 2px)}sd-content-editor[sd-inline=true]{display:inline-block}sd-content-editor[sd-inline=true]>._editor{display:inline-block;width:auto;vertical-align:top}sd-content-editor[sd-inset=true]>._editor{border:none;border-radius:0}sd-content-editor[sd-inset=true][sd-disabled=true]>._editor{background:var(--control-color)!important;color:var(--text-trans-default)}sd-content-editor[sd-inset=true]>._editor{min-height:calc(var(--gap-sm) * 2 + var(--font-size-default) * var(--line-height-strip-unit))}sd-content-editor[sd-inset=true][sd-size=sm]>._editor{min-height:calc(var(--gap-xs) * 2 + var(--font-size-default) * var(--line-height-strip-unit))}sd-content-editor[sd-inset=true][sd-size=lg]>._editor{min-height:calc(var(--gap-default) * 2 + var(--font-size-default) * var(--line-height-strip-unit))}sd-content-editor[sd-inset=true]>._editor:focus{outline:1px solid var(--theme-primary-default);outline-offset:-1px}\n"] }]
    }], () => [], null); })();
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassDebugInfo(SdContentEditorControl, { className: "SdContentEditorControl", filePath: "packages/sd-angular/src/controls/sd-content-editor.control.ts", lineNumber: 143 }); })();
//# sourceMappingURL=sd-content-editor.control.js.map