import { ChangeDetectionStrategy, Component, contentChild, ElementRef, HostListener, input, model, viewChild, ViewEncapsulation, } from "@angular/core";
import { $effect } from "../utils/bindings/$effect";
import { injectElementRef } from "../utils/injections/inject-element-ref";
import { transformBoolean } from "../utils/type-tramsforms";
import { SdDropdownPopupControl } from "./sd-dropdown-popup.control";
import * as i0 from "@angular/core";
const _c0 = ["contentEl"];
const _c1 = ["*", [["sd-dropdown-popup"]]];
const _c2 = ["*", "sd-dropdown-popup"];
export class SdDropdownControl {
    constructor() {
        this._elRef = injectElementRef();
        this.open = model(false);
        this.disabled = input(false, { transform: transformBoolean });
        this.contentClass = input();
        this.contentStyle = input();
        this.contentElRef = viewChild.required("contentEl", {
            read: ElementRef,
        });
        this.popupElRef = contentChild.required(SdDropdownPopupControl, {
            read: ElementRef,
        });
        $effect(() => {
            if (this.open()) {
                document.body.appendChild(this.popupElRef().nativeElement);
                requestAnimationFrame(() => {
                    const contentEl = this.contentElRef().nativeElement;
                    const popupEl = this.popupElRef().nativeElement;
                    const windowOffset = contentEl.getRelativeOffset(window.document.body);
                    const isPlaceBottom = window.innerHeight < windowOffset.top * 2;
                    const isPlaceRight = window.innerWidth < windowOffset.left * 2;
                    Object.assign(popupEl.style, {
                        top: isPlaceBottom ? "" : windowOffset.top + contentEl.offsetHeight + 2 + "px",
                        bottom: isPlaceBottom ? window.innerHeight - windowOffset.top + "px" : "",
                        left: isPlaceRight ? "" : windowOffset.left + "px",
                        right: isPlaceRight
                            ? window.innerWidth - windowOffset.left - contentEl.offsetWidth + "px"
                            : "",
                        minWidth: contentEl.offsetWidth + "px",
                        opacity: "1",
                        pointerEvents: "auto",
                        transform: "none",
                    });
                });
            }
            else {
                const contentEl = this.contentElRef().nativeElement;
                const popupEl = this.popupElRef().nativeElement;
                if (popupEl.matches(":focus, :has(*:focus)")) {
                    contentEl.focus();
                }
                Object.assign(popupEl.style, {
                    top: "",
                    bottom: "",
                    left: "",
                    right: "",
                    minWidth: "",
                    opacity: "",
                    pointerEvents: "",
                    transform: "",
                });
                popupEl.remove();
            }
        });
    }
    _openPopup() {
        if (this.open())
            return;
        if (this.disabled())
            return;
        this.open.set(true);
    }
    _closePopup() {
        if (!this.open())
            return;
        this.open.set(false);
    }
    onDocumentScrollCapture(event) {
        if (this._elRef.nativeElement.findParent(event.target)) {
            const contentEl = this.contentElRef().nativeElement;
            const popupEl = this.popupElRef().nativeElement;
            const windowOffset = contentEl.getRelativeOffset(window.document.body);
            if (window.innerHeight < windowOffset.top * 2) {
                Object.assign(popupEl.style, {
                    top: "",
                    bottom: window.innerHeight - windowOffset.top + "px",
                    left: windowOffset.left + "px",
                });
            }
            else {
                Object.assign(popupEl.style, {
                    top: windowOffset.top + this.contentElRef().nativeElement.offsetHeight + "px",
                    bottom: "",
                    left: windowOffset.left + "px",
                });
            }
        }
    }
    onContentClick() {
        if (this.open()) {
            this._closePopup();
        }
        else {
            this._openPopup();
        }
    }
    onContentKeydown(event) {
        if (!event.ctrlKey && !event.altKey && event.key === "ArrowDown") {
            if (!this.open()) {
                event.preventDefault();
                event.stopPropagation();
                this._openPopup();
            }
            else {
                const popupEl = this.popupElRef().nativeElement;
                const focusableFirst = popupEl.findFocusableFirst();
                if (focusableFirst) {
                    event.preventDefault();
                    event.stopPropagation();
                    focusableFirst.focus();
                }
            }
        }
        if (!event.ctrlKey && !event.altKey && event.key === "ArrowUp") {
            if (this.open()) {
                event.preventDefault();
                event.stopPropagation();
                this._closePopup();
            }
        }
        if (!event.ctrlKey && !event.altKey && event.key === " ") {
            event.preventDefault();
            event.stopPropagation();
            if (this.open()) {
                this._closePopup();
            }
            else {
                this._openPopup();
            }
        }
        if (!event.ctrlKey && !event.altKey && event.key === "Escape") {
            if (this.open()) {
                event.preventDefault();
                event.stopPropagation();
                this._closePopup();
            }
        }
    }
    onPopupKeydown(event) {
        if (!event.ctrlKey && !event.altKey && event.key === "Escape") {
            if (this.open()) {
                event.preventDefault();
                event.stopPropagation();
                this._closePopup();
            }
        }
    }
    onDocumentMouseover(event) {
        this._mouseoverEl = event.target;
    }
    onBlurCapture(event) {
        const contentEl = this.contentElRef().nativeElement;
        const popupEl = this.popupElRef().nativeElement;
        const relatedTarget = event.relatedTarget;
        if (relatedTarget != null &&
            (relatedTarget === contentEl ||
                relatedTarget === popupEl ||
                relatedTarget.findParent(contentEl) ||
                relatedTarget.findParent(popupEl))) {
            return;
        }
        if (relatedTarget == null &&
            this._mouseoverEl instanceof HTMLElement &&
            (this._mouseoverEl.findParent(contentEl) || this._mouseoverEl.findParent(popupEl))) {
            const focusableFirst = popupEl.findFocusableFirst();
            if (focusableFirst) {
                popupEl.findFocusableFirst()?.focus();
            }
            else {
                contentEl.focus();
            }
        }
        if (this.open()) {
            this._closePopup();
        }
    }
    static { this.ɵfac = function SdDropdownControl_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdDropdownControl)(); }; }
    static { this.ɵcmp = /*@__PURE__*/ i0.ɵɵdefineComponent({ type: SdDropdownControl, selectors: [["sd-dropdown"]], contentQueries: function SdDropdownControl_ContentQueries(rf, ctx, dirIndex) { if (rf & 1) {
            i0.ɵɵcontentQuerySignal(dirIndex, ctx.popupElRef, SdDropdownPopupControl, 5, ElementRef);
        } if (rf & 2) {
            i0.ɵɵqueryAdvance();
        } }, viewQuery: function SdDropdownControl_Query(rf, ctx) { if (rf & 1) {
            i0.ɵɵviewQuerySignal(ctx.contentElRef, _c0, 5, ElementRef);
        } if (rf & 2) {
            i0.ɵɵqueryAdvance();
        } }, hostVars: 1, hostBindings: function SdDropdownControl_HostBindings(rf, ctx) { if (rf & 1) {
            i0.ɵɵlistener("scroll.capture", function SdDropdownControl_scroll_capture_HostBindingHandler($event) { return ctx.onDocumentScrollCapture($event); }, false, i0.ɵɵresolveDocument)("mouseover", function SdDropdownControl_mouseover_HostBindingHandler($event) { return ctx.onDocumentMouseover($event); }, false, i0.ɵɵresolveDocument)("blur.capture", function SdDropdownControl_blur_capture_HostBindingHandler($event) { return ctx.onBlurCapture($event); }, false, i0.ɵɵresolveDocument);
        } if (rf & 2) {
            i0.ɵɵattribute("sd-disabled", ctx.disabled());
        } }, inputs: { open: [1, "open"], disabled: [1, "disabled"], contentClass: [1, "contentClass"], contentStyle: [1, "contentStyle"] }, outputs: { open: "openChange" }, ngContentSelectors: _c2, decls: 4, vars: 5, consts: [["contentEl", ""], [1, "_sd-dropdown-control", 3, "click", "keydown"]], template: function SdDropdownControl_Template(rf, ctx) { if (rf & 1) {
            const _r1 = i0.ɵɵgetCurrentView();
            i0.ɵɵprojectionDef(_c1);
            i0.ɵɵelementStart(0, "div", 1, 0);
            i0.ɵɵlistener("click", function SdDropdownControl_Template_div_click_0_listener() { i0.ɵɵrestoreView(_r1); return i0.ɵɵresetView(ctx.onContentClick()); })("keydown", function SdDropdownControl_Template_div_keydown_0_listener($event) { i0.ɵɵrestoreView(_r1); return i0.ɵɵresetView(ctx.onContentKeydown($event)); });
            i0.ɵɵprojection(2);
            i0.ɵɵelementEnd();
            i0.ɵɵprojection(3, 1);
        } if (rf & 2) {
            i0.ɵɵstyleMap(ctx.contentStyle());
            i0.ɵɵclassMap(ctx.contentClass());
            i0.ɵɵattribute("tabindex", ctx.disabled() ? undefined : "0");
        } }, encapsulation: 2, changeDetection: 0 }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdDropdownControl, [{
        type: Component,
        args: [{
                selector: "sd-dropdown",
                changeDetection: ChangeDetectionStrategy.OnPush,
                encapsulation: ViewEncapsulation.None,
                standalone: true,
                imports: [],
                template: `
    <div
      #contentEl
      class="_sd-dropdown-control"
      [attr.tabindex]="disabled() ? undefined : '0'"
      [class]="contentClass()"
      [style]="contentStyle()"
      (click)="onContentClick()"
      (keydown)="onContentKeydown($event)"
    >
      <ng-content />
    </div>
    <ng-content select="sd-dropdown-popup" />
  `,
                host: {
                    "[attr.sd-disabled]": "disabled()",
                },
            }]
    }], () => [], { onDocumentScrollCapture: [{
            type: HostListener,
            args: ["document:scroll.capture", ["$event"]]
        }], onDocumentMouseover: [{
            type: HostListener,
            args: ["document:mouseover", ["$event"]]
        }], onBlurCapture: [{
            type: HostListener,
            args: ["document:blur.capture", ["$event"]]
        }] }); })();
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassDebugInfo(SdDropdownControl, { className: "SdDropdownControl", filePath: "packages/sd-angular/src/controls/sd-dropdown.control.ts", lineNumber: 41 }); })();
//# sourceMappingURL=sd-dropdown.control.js.map