import { ChangeDetectionStrategy, Component, forwardRef, HostListener, inject, input, ViewEncapsulation, } from "@angular/core";
import { SdKanbanBoardControl } from "./sd-kanban-board.control";
import { SdCardControl } from "./sd-card.control";
import { SdKanbanLaneControl } from "./sd-kanban-lane.control";
import { injectElementRef } from "../utils/injections/inject-element-ref";
import { SdEventsDirective } from "../directives/sd-events.directive";
import { NumberUtils } from "@simplysm/sd-core-common";
import { transformBoolean } from "../utils/type-tramsforms";
import { $computed } from "../utils/bindings/$computed";
import { $signal } from "../utils/bindings/$signal";
import * as i0 from "@angular/core";
const _c0 = ["*"];
export class SdKanbanControl {
    constructor() {
        this._boardControl = inject(forwardRef(() => SdKanbanBoardControl));
        this._laneControl = inject(forwardRef(() => SdKanbanLaneControl));
        this._elRef = injectElementRef();
        this.value = input.required();
        this.laneValue = $computed(() => this._laneControl.value());
        this.selectable = input(false, { transform: transformBoolean });
        this.draggable = input(false, { transform: transformBoolean });
        this.selected = $computed(() => this._boardControl.selectedValues().includes(this.value()));
        this.dragKanban = $computed(() => this._boardControl.dragKanban());
        this.contentClass = input();
        this.dragOvered = $signal(false);
        this.heightOnDrag = $signal(0);
        this.cardHeight = $signal(0);
    }
    onClick(event) {
        if (event.shiftKey) {
            if (!this.selectable())
                return;
            event.preventDefault();
            event.stopPropagation();
            this._boardControl.selectedValues.update((v) => {
                const r = [...v];
                if (v.includes(this.value())) {
                    r.remove(this.value());
                }
                else {
                    r.push(this.value());
                }
                return r;
            });
        }
    }
    onCardResize(event) {
        const marginBottom = getComputedStyle(event.target).marginBottom;
        this.cardHeight.set(event.target.clientHeight + (NumberUtils.parseInt(marginBottom)
            ?? 0));
    }
    onCardDragStart() {
        if (!this.draggable())
            return;
        this.heightOnDrag.set(this._elRef.nativeElement.offsetHeight);
        this._boardControl.dragKanban.set(this);
    }
    onDragOver(event) {
        if (this._boardControl.dragKanban() == null)
            return;
        event.preventDefault();
        event.stopPropagation();
        this.dragOvered.set(true);
    }
    onDragLeave(event) {
        event.preventDefault();
        event.stopPropagation();
        this.dragOvered.set(false);
    }
    onDragDrop(event) {
        if (this._boardControl.dragKanban() == null)
            return;
        this.dragOvered.set(false);
        event.preventDefault();
        event.stopPropagation();
        this._boardControl.onDropTo(this);
    }
    onDocumentDrop() {
        this.dragOvered.set(false);
    }
    static { this.ɵfac = function SdKanbanControl_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdKanbanControl)(); }; }
    static { this.ɵcmp = /*@__PURE__*/ i0.ɵɵdefineComponent({ type: SdKanbanControl, selectors: [["sd-kanban"]], hostVars: 3, hostBindings: function SdKanbanControl_HostBindings(rf, ctx) { if (rf & 1) {
            i0.ɵɵlistener("click", function SdKanbanControl_click_HostBindingHandler($event) { return ctx.onClick($event); })("drop.capture", function SdKanbanControl_drop_capture_HostBindingHandler() { return ctx.onDocumentDrop(); }, false, i0.ɵɵresolveDocument);
        } if (rf & 2) {
            i0.ɵɵattribute("sd-dragging-this", ctx.dragKanban() === ctx)("sd-dragging", ctx.dragKanban() != null)("sd-drag-over", ctx.dragOvered());
        } }, inputs: { value: [1, "value"], selectable: [1, "selectable"], draggable: [1, "draggable"], contentClass: [1, "contentClass"] }, ngContentSelectors: _c0, decls: 4, vars: 9, consts: [[1, "_drag-position", 3, "dragover", "dragleave", "drop"], [1, "_drop-position"], [3, "dragstart", "sdResize", "draggable"]], template: function SdKanbanControl_Template(rf, ctx) { if (rf & 1) {
            i0.ɵɵprojectionDef();
            i0.ɵɵelementStart(0, "div", 0);
            i0.ɵɵlistener("dragover", function SdKanbanControl_Template_div_dragover_0_listener($event) { return ctx.onDragOver($event); })("dragleave", function SdKanbanControl_Template_div_dragleave_0_listener($event) { return ctx.onDragLeave($event); })("drop", function SdKanbanControl_Template_div_drop_0_listener($event) { return ctx.onDragDrop($event); });
            i0.ɵɵelementEnd();
            i0.ɵɵelement(1, "div", 1);
            i0.ɵɵelementStart(2, "sd-card", 2);
            i0.ɵɵlistener("dragstart", function SdKanbanControl_Template_sd_card_dragstart_2_listener() { return ctx.onCardDragStart(); })("sdResize", function SdKanbanControl_Template_sd_card_sdResize_2_listener($event) { return ctx.onCardResize($event); });
            i0.ɵɵprojection(3);
            i0.ɵɵelementEnd();
        } if (rf & 2) {
            let tmp_1_0;
            i0.ɵɵstyleProp("height", ctx.cardHeight() + "px");
            i0.ɵɵadvance();
            i0.ɵɵstyleProp("height", ctx.dragOvered() ? ((tmp_1_0 = (tmp_1_0 = ctx.dragKanban()) == null ? null : tmp_1_0.heightOnDrag()) !== null && tmp_1_0 !== undefined ? tmp_1_0 : 0) + "px" : "0px")("display", ctx.dragKanban() ? "block" : "none");
            i0.ɵɵadvance();
            i0.ɵɵclassMap(ctx.contentClass());
            i0.ɵɵproperty("draggable", ctx.draggable());
        } }, dependencies: [SdCardControl, SdEventsDirective], styles: ["sd-kanban{position:relative;display:block}sd-kanban[sd-dragging-this=true]{display:none}sd-kanban>._drag-position{position:absolute;top:0;left:0;right:0;pointer-events:none}sd-kanban[sd-dragging=true]>._drag-position{pointer-events:auto}sd-kanban[sd-dragging=true]>sd-card{pointer-events:none}sd-kanban>._drop-position{border-radius:var(--border-radius-default);background:var(--trans-light);height:0;margin-bottom:0;visibility:hidden;transition:.1s linear;transition-property:height,margin-bottom,visibility}sd-kanban[sd-drag-over=true]>._drop-position{margin-bottom:var(--gap-lg);visibility:visible}sd-kanban>sd-card{white-space:normal;user-select:none;cursor:pointer;margin-bottom:var(--gap-lg)}\n"], encapsulation: 2, changeDetection: 0 }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdKanbanControl, [{
        type: Component,
        args: [{ selector: "sd-kanban", changeDetection: ChangeDetectionStrategy.OnPush, encapsulation: ViewEncapsulation.None, standalone: true, imports: [SdCardControl, SdEventsDirective], template: `
    <div
      class="_drag-position"
      [style.height]="cardHeight() + 'px'"
      (dragover)="onDragOver($event)"
      (dragleave)="onDragLeave($event)"
      (drop)="onDragDrop($event)"
    ></div>
    <div
      class="_drop-position"
      [style.height]="dragOvered() ? (dragKanban()?.heightOnDrag() ?? 0) + 'px' : '0px'"
      [style.display]="dragKanban() ? 'block' : 'none'"
    ></div>
    <sd-card
      [class]="contentClass()"
      [draggable]="draggable()"
      (dragstart)="onCardDragStart()"
      (sdResize)="onCardResize($event)"
    >
      <ng-content></ng-content>
    </sd-card>
  `, host: {
                    "[attr.sd-dragging-this]": "dragKanban() === this",
                    "[attr.sd-dragging]": "dragKanban() != null",
                    "[attr.sd-drag-over]": "dragOvered()",
                }, styles: ["sd-kanban{position:relative;display:block}sd-kanban[sd-dragging-this=true]{display:none}sd-kanban>._drag-position{position:absolute;top:0;left:0;right:0;pointer-events:none}sd-kanban[sd-dragging=true]>._drag-position{pointer-events:auto}sd-kanban[sd-dragging=true]>sd-card{pointer-events:none}sd-kanban>._drop-position{border-radius:var(--border-radius-default);background:var(--trans-light);height:0;margin-bottom:0;visibility:hidden;transition:.1s linear;transition-property:height,margin-bottom,visibility}sd-kanban[sd-drag-over=true]>._drop-position{margin-bottom:var(--gap-lg);visibility:visible}sd-kanban>sd-card{white-space:normal;user-select:none;cursor:pointer;margin-bottom:var(--gap-lg)}\n"] }]
    }], null, { onClick: [{
            type: HostListener,
            args: ["click", ["$event"]]
        }], onDocumentDrop: [{
            type: HostListener,
            args: ["document:drop.capture"]
        }] }); })();
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassDebugInfo(SdKanbanControl, { className: "SdKanbanControl", filePath: "packages/sd-angular/src/controls/sd-kanban.control.ts", lineNumber: 114 }); })();
//# sourceMappingURL=sd-kanban.control.js.map