import { inject, Injectable } from "@angular/core";
import { EventManagerPlugin } from "@angular/platform-browser";
import { DOCUMENT } from "@angular/common";
import * as i0 from "@angular/core";
export class SdResizeEventPlugin extends EventManagerPlugin {
    constructor() {
        super(inject(DOCUMENT));
    }
    supports(eventName) {
        return eventName === "sdResize";
    }
    addEventListener(element, eventName, handler) {
        let prevWidth = 0;
        let prevHeight = 0;
        const intersectionObserver = new IntersectionObserver(([entry]) => {
            if (!entry.isIntersecting)
                return;
            const contentRect = entry.boundingClientRect;
            const heightChanged = contentRect.height !== prevHeight;
            const widthChanged = contentRect.width !== prevWidth;
            prevHeight = contentRect.height;
            prevWidth = contentRect.width;
            handler({
                heightChanged,
                widthChanged,
                target: entry.target,
                contentRect: entry.boundingClientRect,
            });
            intersectionObserver.unobserve(entry.target);
        });
        intersectionObserver.observe(element);
        const resizeObserver = new ResizeObserver(([entry]) => {
            const contentRect = entry.contentRect;
            const heightChanged = contentRect.height !== prevHeight;
            const widthChanged = contentRect.width !== prevWidth;
            prevHeight = contentRect.height;
            prevWidth = contentRect.width;
            handler({
                heightChanged,
                widthChanged,
                target: entry.target,
                contentRect: entry.contentRect,
            });
        });
        resizeObserver.observe(element);
        return () => {
            resizeObserver.disconnect();
        };
    }
    static { this.ɵfac = function SdResizeEventPlugin_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdResizeEventPlugin)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdResizeEventPlugin, factory: SdResizeEventPlugin.ɵfac, providedIn: null }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdResizeEventPlugin, [{
        type: Injectable,
        args: [{ providedIn: null }]
    }], () => [], null); })();
//# sourceMappingURL=sd-resize.event-plugin.js.map