import { ApplicationRef, createComponent, inject, Injectable, Injector, } from "@angular/core";
import { SdModalControl } from "../controls/sd-modal.control";
import { $signal } from "../utils/bindings/$signal";
import { SdBusyProvider } from "./sd-busy.provider";
import * as i0 from "@angular/core";
export class SdModalProvider {
    constructor() {
        this._appRef = inject(ApplicationRef);
        this._sdBusy = inject(SdBusyProvider);
        this.modalCount = $signal(0);
    }
    async showAsync(modal, options) {
        let isFirstOpen = true;
        this._sdBusy.globalBusyCount.update((v) => v + 1);
        return await new Promise((resolve, reject) => {
            try {
                //-- Provider
                const provider = new SdActivatedModalProvider();
                //-- Content component
                const compRef = createComponent(modal.type, {
                    environmentInjector: this._appRef.injector,
                    elementInjector: Injector.create({
                        parent: this._appRef.injector,
                        providers: [{ provide: SdActivatedModalProvider, useValue: provider }],
                    }),
                });
                for (const inputKey in modal.inputs) {
                    compRef.setInput(inputKey, modal.inputs[inputKey]);
                }
                provider.contentComponent.set(compRef.instance);
                const compEl = compRef.location.nativeElement;
                //-- Modal component
                const modalRef = createComponent(SdModalControl, {
                    environmentInjector: this._appRef.injector,
                    projectableNodes: [[compEl]],
                    elementInjector: Injector.create({
                        parent: this._appRef.injector,
                        providers: [{ provide: SdActivatedModalProvider, useValue: provider }],
                    }),
                });
                modalRef.setInput("title", modal.title);
                modalRef.setInput("key", options?.key);
                modalRef.setInput("hideHeader", options?.hideHeader ?? false);
                modalRef.setInput("hideCloseButton", options?.hideCloseButton ?? false);
                modalRef.setInput("useCloseByBackdrop", options?.useCloseByBackdrop ?? false);
                modalRef.setInput("useCloseByEscapeKey", options?.useCloseByEscapeKey ?? false);
                modalRef.setInput("float", options?.float ?? false);
                modalRef.setInput("minHeightPx", options?.minHeightPx);
                modalRef.setInput("minWidthPx", options?.minWidthPx);
                modalRef.setInput("resizable", options?.resizable ?? true);
                modalRef.setInput("movable", options?.movable ?? true);
                modalRef.setInput("headerStyle", options?.headerStyle);
                modalRef.setInput("fill", options?.fill ?? false);
                provider.modalComponent.set(modalRef.instance);
                const modalEl = modalRef.location.nativeElement;
                modalRef.instance.open.subscribe((value) => {
                    if (!provider.canDeactivefn())
                        return;
                    if (!value) {
                        modalRef.setInput("open", value);
                        compRef.instance.close.emit(undefined);
                    }
                    else {
                        modalRef.setInput("open", value);
                    }
                });
                const prevActiveElement = document.activeElement;
                compRef.instance.close.subscribe((value) => {
                    if (value != null && !provider.canDeactivefn())
                        return;
                    resolve(value);
                    modalEl.addEventListener("transitionend", () => {
                        compRef.destroy();
                        modalRef.destroy();
                    });
                    modalRef.setInput("open", false);
                    this.modalCount.update((v) => v - 1);
                    if (prevActiveElement) {
                        prevActiveElement.focus();
                    }
                });
                const rootEl = this._appRef.components[0].location.nativeElement;
                rootEl.appendChild(modalEl);
                this._appRef.attachView(compRef.hostView);
                this._appRef.attachView(modalRef.hostView);
                //-- open
                this.modalCount.update((v) => v + 1);
                requestAnimationFrame(() => {
                    modalRef.setInput("open", true);
                    if (isFirstOpen) {
                        isFirstOpen = false;
                        this._sdBusy.globalBusyCount.update((v) => v - 1);
                    }
                    requestAnimationFrame(() => {
                        if (options?.noFirstControlFocusing) {
                            modalRef.instance.dialogElRef().nativeElement.focus();
                        }
                        else {
                            (compRef.location.nativeElement.findFocusableFirst() ??
                                modalRef.instance.dialogElRef().nativeElement).focus();
                        }
                    });
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    static { this.ɵfac = function SdModalProvider_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdModalProvider)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdModalProvider, factory: SdModalProvider.ɵfac, providedIn: "root" }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdModalProvider, [{
        type: Injectable,
        args: [{ providedIn: "root" }]
    }], null, null); })();
export class SdActivatedModalProvider {
    constructor() {
        this.modalComponent = $signal();
        this.contentComponent = $signal();
        this.canDeactivefn = () => true;
    }
    static { this.ɵfac = function SdActivatedModalProvider_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdActivatedModalProvider)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdActivatedModalProvider, factory: SdActivatedModalProvider.ɵfac }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdActivatedModalProvider, [{
        type: Injectable
    }], null, null); })();
//# sourceMappingURL=sd-modal.provider.js.map