import { afterNextRender, ApplicationRef, createComponent, Directive, inject, Injectable, input, } from "@angular/core";
import { jsPDF } from "jspdf";
import * as htmlToImage from "html-to-image";
import * as i0 from "@angular/core";
export class SdPrintProvider {
    constructor() {
        this._appRef = inject(ApplicationRef);
    }
    async printAsync(printType, params, options) {
        await new Promise((resolve) => {
            //-- comp
            const compRef = createComponent(printType, {
                environmentInjector: this._appRef.injector,
            });
            compRef.setInput("params", params);
            const compEl = compRef.location.nativeElement;
            compEl.classList.add("_sd-print-template");
            //-- style
            const styleEl = document.createElement("style");
            styleEl.innerHTML = `   
  @page {
      size: ${options?.size ?? "A4 auto"}; 
      margin: ${options?.margin ?? "0"};
  }
  body > ._sd-print-template { display: none; }
  @media print
  {
      html, body { -webkit-print-color-adjust: exact; background: white; }
      body > * { display: none !important; }
      body > ._sd-print-template { display: block !important; }
  }`;
            compRef.instance.print = () => {
                afterNextRender(() => {
                    document.body.appendChild(compEl);
                    document.head.appendChild(styleEl);
                    requestAnimationFrame(() => {
                        window.print();
                        compRef.destroy();
                        styleEl.remove();
                        resolve();
                    });
                }, { injector: compRef.injector });
            };
            this._appRef.attachView(compRef.hostView);
        });
    }
    async getPdfBufferAsync(printType, params, options) {
        return await new Promise((resolve, reject) => {
            try {
                //-- comp
                const compRef = createComponent(printType, {
                    environmentInjector: this._appRef.injector,
                });
                compRef.setInput("params", params);
                const compEl = compRef.location.nativeElement;
                compEl.classList.add("_sd-print-template");
                //-- style
                const styleEl = document.createElement("style");
                styleEl.innerHTML = `html, body { overflow: hidden }`;
                compRef.instance.print = () => {
                    afterNextRender({
                        read: async () => {
                            try {
                                document.body.appendChild(compEl);
                                document.head.appendChild(styleEl);
                                const doc = new jsPDF(options?.orientation ?? "p", "pt", "a4");
                                doc.deletePage(1);
                                let els = compEl.findAll("sd-print-page");
                                els = els.length > 0 ? els : [compEl];
                                for (const el of els) {
                                    const canvas = await htmlToImage.toCanvas(el, {
                                        backgroundColor: "white",
                                        pixelRatio: 4,
                                    });
                                    const orientation = el.getAttribute("sd-orientation");
                                    doc.addPage("a4", orientation ?? "p").addImage({
                                        imageData: canvas,
                                        x: 0,
                                        y: 0,
                                        ...(orientation === "landscape"
                                            ? {
                                                height: 841.89,
                                                width: 595.28,
                                            }
                                            : {
                                                width: 595.28,
                                                height: 841.89,
                                            }),
                                    });
                                }
                                const arrayBuffer = doc.output("arraybuffer");
                                compRef.destroy();
                                styleEl.remove();
                                resolve(Buffer.from(arrayBuffer));
                            }
                            catch (err) {
                                reject(err);
                            }
                        },
                    }, { injector: compRef.injector });
                };
                this._appRef.attachView(compRef.hostView);
            }
            catch (err) {
                reject(err);
            }
        });
    }
    static { this.ɵfac = function SdPrintProvider_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdPrintProvider)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdPrintProvider, factory: SdPrintProvider.ɵfac, providedIn: "root" }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdPrintProvider, [{
        type: Injectable,
        args: [{ providedIn: "root" }]
    }], null, null); })();
export class SdPrintTemplateBase {
    constructor() {
        this.params = input.required();
    }
    print() {
        throw new Error("템플릿이 초기화되어있지 않습니다.");
    }
    static { this.ɵfac = function SdPrintTemplateBase_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdPrintTemplateBase)(); }; }
    static { this.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: SdPrintTemplateBase, inputs: { params: [1, "params"] } }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdPrintTemplateBase, [{
        type: Directive
    }], null, null); })();
//# sourceMappingURL=sd-print.provider.js.map