import { ApplicationRef, createComponent, Directive, inject, Injectable, input, } from "@angular/core";
import { SdToastContainerControl } from "../controls/sd-toast-container.control";
import { SdSystemLogProvider } from "./sd-system-log.provider";
import { SdToastControl } from "../controls/sd-toast.control";
import * as i0 from "@angular/core";
export class SdToastProvider {
    constructor() {
        this._appRef = inject(ApplicationRef);
        this._systemLog = inject(SdSystemLogProvider);
        this.alertThemes = [];
        this.overlap = false;
    }
    get containerRef() {
        if (this._containerRef === undefined) {
            const compRef = createComponent(SdToastContainerControl, {
                environmentInjector: this._appRef.injector,
            });
            compRef.setInput("overlap", this.overlap);
            const rootComp = this._appRef.components[0];
            const rootCompEl = rootComp.location.nativeElement;
            rootCompEl.appendChild(compRef.location.nativeElement);
            this._appRef.attachView(compRef.hostView);
            this._containerRef = compRef;
        }
        return this._containerRef;
    }
    async try(fn, messageFn) {
        try {
            return await fn();
        }
        catch (err) {
            if (err instanceof Error) {
                if (messageFn) {
                    this.danger(messageFn(err));
                }
                else {
                    this.danger(err.message);
                }
                await this._systemLog.writeAsync("error", err.stack);
                return undefined;
            }
            else {
                throw err;
            }
        }
    }
    notify(toastType, params, onclose) {
        const compRef = createComponent(toastType, {
            environmentInjector: this._appRef.injector,
        });
        const containerEl = this.containerRef.location.nativeElement;
        const toastRef = createComponent(SdToastControl, {
            environmentInjector: this._appRef.injector,
            projectableNodes: [[compRef.location.nativeElement]],
        });
        const toastEl = toastRef.location.nativeElement;
        containerEl.appendChild(toastEl);
        const close = async (value) => {
            toastEl.addEventListener("transitionend", () => {
                compRef.destroy();
                toastRef.destroy();
            });
            toastRef.setInput("open", false);
            await onclose(value);
        };
        toastRef.instance.close.subscribe(async () => {
            await close();
        });
        this._appRef.attachView(toastRef.hostView);
        compRef.setInput("params", params);
        compRef.instance.close = async (v) => {
            await close(v);
        };
        this._appRef.attachView(compRef.hostView);
        toastRef.setInput("open", true);
        window.setTimeout(() => {
            compRef.destroy();
            toastRef.destroy();
        }, 5000);
    }
    info(message, progress) {
        return this._show("info", message, progress);
    }
    success(message, progress) {
        return this._show("success", message, progress);
    }
    warning(message, progress) {
        return this._show("warning", message, progress);
    }
    danger(message, progress) {
        return this._show("danger", message, progress);
    }
    _show(theme, message, progress) {
        this.beforeShowFn?.(theme);
        if (this.alertThemes.includes(theme)) {
            alert(message);
            return undefined;
        }
        const toastRef = createComponent(SdToastControl, {
            environmentInjector: this._appRef.injector,
        });
        const toastEl = toastRef.location.nativeElement;
        const containerEl = this.containerRef.location.nativeElement;
        if (this.overlap) {
            for (const child of Array.from(containerEl.children)) {
                containerEl.removeChild(child);
            }
        }
        containerEl.appendChild(toastEl);
        this._appRef.attachView(toastRef.hostView);
        toastEl.findAll("._sd-toast-message")[0].innerText = message;
        toastRef.setInput("useProgress", progress ?? false);
        toastRef.setInput("progress", 0);
        // repaint
        // eslint-disable-next-line @typescript-eslint/no-unused-expressions
        containerEl.offsetHeight;
        toastRef.setInput("open", true);
        toastRef.setInput("theme", theme);
        if (progress) {
            return {
                progress: (percent) => {
                    toastRef.setInput("progress", percent);
                    if (percent >= 100) {
                        this._closeAfterTime(toastRef, 1000);
                    }
                },
                message: (msg) => {
                    toastEl.findAll("._sd-toast-message")[0].innerText = msg;
                },
            };
        }
        else {
            this._closeAfterTime(toastRef, 3000);
            return undefined;
        }
    }
    _closeAfterTime(toastRef, ms) {
        const toastEl = toastRef.location.nativeElement;
        window.setTimeout(() => {
            if (toastEl.matches(":hover")) {
                this._closeAfterTime(toastRef, ms);
            }
            else {
                toastEl.addEventListener("transitionend", () => {
                    toastRef.destroy();
                });
                toastRef.setInput("open", false);
            }
        }, ms);
    }
    static { this.ɵfac = function SdToastProvider_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdToastProvider)(); }; }
    static { this.ɵprov = /*@__PURE__*/ i0.ɵɵdefineInjectable({ token: SdToastProvider, factory: SdToastProvider.ɵfac, providedIn: "root" }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdToastProvider, [{
        type: Injectable,
        args: [{ providedIn: "root" }]
    }], null, null); })();
export class SdToastBase {
    constructor() {
        this.params = input.required();
    }
    close(value) {
        throw new Error("초기화되어있지 않습니다.");
    }
    static { this.ɵfac = function SdToastBase_Factory(__ngFactoryType__) { return new (__ngFactoryType__ || SdToastBase)(); }; }
    static { this.ɵdir = /*@__PURE__*/ i0.ɵɵdefineDirective({ type: SdToastBase, inputs: { params: [1, "params"] } }); }
}
(() => { (typeof ngDevMode === "undefined" || ngDevMode) && i0.ɵsetClassMetadata(SdToastBase, [{
        type: Directive
    }], null, null); })();
//# sourceMappingURL=sd-toast.provider.js.map