import { ObjectUtils } from "@simplysm/sd-core-common";
const ORIGIN_SNAPSHOT = Symbol();
export function $arr(sig) {
    return {
        insert(i, item) {
            if (!("update" in sig))
                throw new Error("Readonly signal does not support insert.");
            sig.update((v) => {
                const r = [...v];
                r.insert(i, item);
                return r;
            });
        },
        remove(itemOrFn) {
            if (!("update" in sig))
                throw new Error("Readonly signal does not support remove.");
            sig.update((v) => {
                const r = [...v];
                r.remove(itemOrFn);
                return r;
            });
        },
        toggle(value) {
            if (!("update" in sig))
                throw new Error("Readonly signal does not support toggle.");
            sig.update((v) => {
                if (v.includes(value)) {
                    return v.filter((item) => item !== value);
                }
                else {
                    return [...v, value];
                }
                return v;
            });
        },
        snapshot(keyPropName) {
            sig[ORIGIN_SNAPSHOT] = {
                keyPropName,
                snapshot: ObjectUtils.clone(sig()).toMap((item) => keyPropName === "$item" ? item : item[keyPropName]),
            };
        },
        changed(item) {
            if (sig[ORIGIN_SNAPSHOT] == null)
                return false;
            const orgItemMap = sig[ORIGIN_SNAPSHOT].snapshot;
            const keyPropName = sig[ORIGIN_SNAPSHOT].keyPropName;
            if (item[keyPropName] == null)
                return true;
            const orgItem = orgItemMap.get(item[keyPropName]);
            return !ObjectUtils.equal(orgItem, item);
        },
        diffs() {
            if (sig[ORIGIN_SNAPSHOT] == null)
                return [];
            const orgItemMap = sig[ORIGIN_SNAPSHOT].snapshot;
            const keyPropName = sig[ORIGIN_SNAPSHOT].keyPropName;
            return sig().oneWayDiffs(orgItemMap, keyPropName);
        },
        get origin() {
            return sig[ORIGIN_SNAPSHOT]?.snapshot ?? new Map();
        },
    };
}
//# sourceMappingURL=$arr.js.map