import { $computed } from "../bindings/$computed";
export class SdSelectionManager {
    constructor(_options) {
        this._options = _options;
        this._selectableItems = $computed(() => this._options.displayItems().filter((item) => this.getSelectable(item) === true));
        this.hasSelectable = $computed(() => this._selectableItems().length > 0);
        this.isAllSelected = $computed(() => this.hasSelectable() &&
            this._selectableItems().every((item) => this._options.selectedItems().includes(item)));
    }
    getBeforeChangeFn(item) {
        return () => this.getSelectable(item) === true;
    }
    getSelectable(item) {
        const fn = this._options.getItemSelectableFn();
        if (!fn)
            return true;
        const result = fn(item);
        if (result === true)
            return true;
        return result === false ? undefined : result;
    }
    toggleAll() {
        if (this.isAllSelected()) {
            this._options.selectedItems.update((v) => v.filter((item) => !this._selectableItems().includes(item)));
        }
        else {
            this._options.selectedItems.update((v) => [...v, ...this._selectableItems()].distinct());
        }
    }
    select(item) {
        if (this._options.selectMode() === "single") {
            this._options.selectedItems.set([item]);
        }
        else {
            this._options.selectedItems.update((list) => [...list, item].distinct());
        }
    }
    toggle(item) {
        if (this._options.selectedItems().includes(item)) {
            this._options.selectedItems.update((list) => list.filter((i) => i !== item));
        }
        else if (this._options.selectMode() === "single") {
            this._options.selectedItems.set([item]);
        }
        else {
            this._options.selectedItems.update((list) => [...list, item].distinct());
        }
    }
}
//# sourceMappingURL=sd-selection-manager.js.map